/*
  Top 10, a racing simulator
  Copyright (C) 2003,2005  Johann Deneux
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Authors can be contacted at following electronic addresses:
  Johann Deneux: johann.deneux@it.uu.se
*/

#ifndef TOP10_MATHRAY_HH
#define TOP10_MATHRAY_HH

#include "Vertex.hh"
#include "Matrix.hh"
#include <cassert>

namespace top10 {
  namespace math {

    class Ray {
    public:
      Ray();
      
      inline bool isFinite() const { return length < 0.0; }
      inline void setInfinite() { length = -1.0; }
      inline void setLength(double l) { assert(l>=0); length = l; }
      inline double getLength() const { return length; }
      inline Vector getOrigin() const { return origin; }
      inline Vector getDirection() const { return dir; }
      inline void setOrigin(Vector o) { origin = o; }
      inline void setDirection(Vector d) { double s = d.size(); assert(s>=SMALL_VECTOR); dir = d/s; }
    
      void transform(const Matrix4& M);
      
    private:
      Vector origin;
      Vector dir;
      double length;
    };

  }
}

#endif
